/*global JQueryEventObject, JQuery*/
/* Searchbox.ts

	Purpose:

	Description:

	History:
		Mon Jul 15 12:43:43 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
(() => {

interface Item {
	id: string;
	content: string;
}

enum UpdateType {
	CONTENTS_CHANGED,
	INTERVAL_ADDED,
	INTERVAL_REMOVED,
}

let _zkf: () => void;

/**
 * A dropdown list that allows users to search and select items.
 * <p>Available in ZK EE.
 *
 * @author rudyhuang
 * @since 9.0.0
 */
zkmax.inp.Searchbox = zk.$extends(zul.Widget, {
	_searchMessage: 'Type to search',
	_multiple: false,
	_open: false,
	_disabled: false,
	_autoclose: false,
	_renderByServer: false,
	_lastSearching: '',

	$init() {
		this.$supers('$init', arguments);
		this._items = [];
		this._selectedUuids = [];
		this._dirtyUuids = {}; // key: string (uuid), value: boolean (selected)
	},

	$define: {
		/**
		 * Returns the placeholder message of the search text field,
		 * or null to use the default message.
		 * @return String
		 */
		/**
		 * Sets the placeholder message of the search text field.
		 * @param String message
		 */
		searchMessage(v: string) {
			let search = this.$n('search') as HTMLInputElement;
			if (search) {
				search.placeholder = v;
			}
		},
		/**
		 * Returns the placeholder text.
		 * @return String
		 */
		/**
		 * Sets the placeholder text that is displayed when selected item is empty.
		 * @param String placeholder
		 */
		placeholder(v: string) {
			let placeholder = this.$n('placeholder');
			if (placeholder) {
				placeholder.innerHTML = v;
			}
		},
		/**
		 * Returns whether multiple selections are allowed.
		 * <p>Default: false.
		 * @return boolean
		 */
		/**
		 * Sets whether multiple selections are allowed.
		 * <p>Notice that, if a model is assigned, it will change the model's
		 * state.
		 * @param boolean multiple
		 */
		multiple: _zkf = function (this: zk.Widget) {
			if (this.desktop) {
				this._redrawItemsFlag = true;
			}
		},
		/**
		 * Returns whether it is disabled.
		 * <p>Default: false.
		 * @return boolean
		 */
		/**
		 * Sets whether it is disabled.
		 * @param boolean disabled
		 */
		disabled(v: boolean) {
			if (this.desktop) {
				jq(this.$n()).toggleClass(this.$s('disabled'))
					.attr('disabled', v ? 'disabled' : null)
					.attr('tabindex', v ? -1 : this._tabindex || 0);
			}
		},
		/**
		 * Returns whether to automatically close the list if a user is selected
		 * any item.
		 * <p>Default: false
		 * @return boolean
		 */
		/**
		 * Sets whether to automatically close the list if a user is selected
		 * any item.
		 * @param boolean autoclose
		 */
		autoclose: null,
		// private properties for component
		placeholderVisible(v: boolean) {
			if (this.desktop) {
				jq(this.$n('placeholder')).css('display', v ? 'inline-block' : 'none');
				jq(this.$n('label')).css('display', !v ? 'inline-block' : 'none');
				this._toggleClearButtonVisible(!v);
			}
		},
		items: _zkf,
		selectedUuids: _zkf,
		renderByServer(v: boolean) {
			if (this.desktop && v) {
				jq(this.$n('cave')).html('');
				if (this._open) {
					this._repositionPopup();
					this._fixBothScrollbar();
				}
			}
		},
		label(v: string) {
			if (this.desktop) {
				this.$n('label').innerHTML = v;
			}
		}
	},

	bind_() {
		this.$supers(zkmax.inp.Searchbox, 'bind_', arguments);
		this.domListen_(this.$n('search'), 'onFocus', '_doSearchFocus')
			.domListen_(this.$n('search'), 'onBlur', '_doSearchBlur');
		jq(this.$n()).on('focusin', this.proxy(this._focusinContent)).on('focusout', this.proxy(this._focusoutContent));
		jq(this.$n('pp')).on('mousedown', `.${this.$s('item')}`, this._doItemMouseDown.bind(this)).on('click', `.${this.$s('item')}`, this._doItemClick.bind(this));
		zWatch.listen({onFloatUp: this, onResponse: this, onSize: this});
		this._toggleClearButtonVisible(!this._placeholderVisible);
	},
	unbind_() {
		if (this._open) {
			this.close();
		}
		zWatch.unlisten({onFloatUp: this, onResponse: this, onSize: this});
		jq(this.$n('pp')).off('click', `.${this.$s('item')}`).off('mousedown', `.${this.$s('item')}`);
		jq(this.$n()).off('focusout', this.proxy(this._focusoutContent)).off('focusin', this.proxy(this._focusinContent));
		this.domUnlisten_(this.$n('search'), 'onBlur', '_doSearchBlur')
			.domUnlisten_(this.$n('search'), 'onFocus', '_doSearchFocus');
		this.$class._stopOnSearching(this);
		this.$supers(zkmax.inp.Searchbox, 'unbind_', arguments);
	},

	domClass_(): string {
		let classes = '';
		if (this._disabled)
			classes += ` ${this.$s('disabled')}`;
		if (this._open)
			classes += ` ${this.$s('open')}`;
		return this.$supers('domClass_', arguments) + classes;
	},
	domAttrs_(): string {
		let attrs: string = this.$supers('domAttrs_', arguments);
		if (this._disabled)
			attrs += ' disabled="disabled"';
		return attrs;
	},

	_doItemClick(evt: JQueryEventObject) {
		this._doSelectItem(evt.currentTarget, this._multiple && evt.shiftKey);
		evt.preventDefault();
		evt.stopPropagation();
	},
	doClick_(evt: zk.Event) {
		if (!this._disabled) {
			if (evt.domTarget == this.$n('clear'))
				this.clearSelection();
			else if (!jq.isAncestor(this.$n('pp'), evt.domTarget))
				this.setOpen(!this._open);
			if (!evt.stopped)
				this.$supers('doClick_', arguments);
		}
	},
	doKeyDown_(evt: zk.Event) {
		if (this._disabled) return;
		switch (evt.key) {
			case 'Escape':
				this.close(true);
				break;
			case 'Tab':
				this.close();
				break;
			case 'Enter':
				this._enterPressed(this._multiple && evt.shiftKey);
				evt.stop();
				break;
			case 'Delete':
			case 'Backspace':
				if (!this.isOpen()) {
					this.clearSelection();
					evt.stop();
				}
				break;
			case 'ArrowUp':
				if (this._doOpenPopup(evt)) return;
				this._shiftSelection(evt, -1);
				break;
			case 'ArrowDown':
				if (this._doOpenPopup(evt)) return;
				this._shiftSelection(evt, 1);
				break;
			case 'PageUp':
				this._shiftSelection(evt, -30);
				break;
			case 'PageDown':
				this._shiftSelection(evt, 30);
				break;
			case 'Home':
				this._shiftSelection(evt, -Infinity);
				break;
			case 'End':
				this._shiftSelection(evt, Infinity);
				break;
		}
		this.$class._startOnSearching(this);
	},
	_doOpenPopup(evt: zk.Event): boolean {
		if (!this._open) {
			this.open();
			evt.stop();
			return true;
		}
		return false;
	},

	_enterPressed(bulk: boolean) {
		let jqCurrentActive = jq(this.$n('cave')).find(`.${this.$s('active')}:visible`);
		if (jqCurrentActive.length > 0)
			this._doSelectItem(jqCurrentActive.get(0), bulk);
	},

	_doSelectItem(item: Element, bulk: boolean) {
		let itemId = item.id,
			jqItem = jq(item),
			selectedClass = this.$s('selected'),
			selected = !jqItem.hasClass(selectedClass);
		if (!this._multiple) {
			jqItem.siblings(`.${selectedClass}`).removeClass(selectedClass);
			this._selectedUuids = [];
		}
		if (bulk && this._lastSelectedItem) {
			let items = jq(this.$n('cave')).children(`.${this.$s('item')}`),
				jqLastSelectedItem = jq(this._lastSelectedItem),
				index0 = jqLastSelectedItem.index(),
				index1 = jqItem.index(),
				selected = jqLastSelectedItem.hasClass(selectedClass),
				bulkItems = items.slice(Math.min(index0, index1), Math.max(index0, index1) + 1);
			bulkItems
				.toggleClass(selectedClass, selected)
				.each((index, element) => {
					let id = element.id;
					this._toggleSelectedUuids(id, selected);
					this._toggleDirtyUuids(id, selected);
				});
		} else {
			jqItem.toggleClass(selectedClass, selected);
			this._toggleSelectedUuids(itemId, selected);
			if (this._multiple) {
				this._toggleDirtyUuids(itemId, selected);
			}
		}
		this._lastSelectedItem = item;
		if (this._autoclose) {
			this.close(true);
		}
	},
	_toggleSelectedUuids(uuid: string, selected: boolean) {
		let selectedUuids: string[] = this._selectedUuids,
			selectedIndex = selectedUuids.indexOf(uuid);
		if (selected) {
			if (selectedIndex == -1)
				selectedUuids.push(uuid);
		} else {
			if (selectedIndex != -1)
				selectedUuids.splice(selectedIndex, 1);
		}
	},
	_toggleDirtyUuids(uuid: string, selected: boolean) {
		let dirtySelected = this._dirtyUuids[uuid];
		if (dirtySelected === undefined) {
			this._dirtyUuids[uuid] = selected;
		} else if (dirtySelected != selected) { // revert
			delete this._dirtyUuids[uuid];
		}
	},

	_shiftSelection(evt: zk.Event, step: number) {
		let isUp = step < 0,
			pos = Math.abs(step) - 1,
			activeClass = this.$s('active'),
			itemClass = this.$s('item'),
			jqCave = jq(this.$n('cave')),
			jqCurrentActive = jqCave.find(`.${activeClass}`),
			jqNextActive: JQuery;

		evt.stop();
		if (!isFinite(pos)) { // Home/End
			jqNextActive = jqCave.children(`.${itemClass}:visible`)[isUp ? 'first' : 'last']();
		} else {
			if (jqCurrentActive.is(':visible')) {
				let siblings = jqCurrentActive[isUp ? 'prevAll' : 'nextAll'](':visible');
				if (pos >= siblings.length) {
					pos = siblings.length - 1;
				}
				jqNextActive = siblings.eq(pos);
			} else {
				// No active visible
				jqNextActive = jqCave.children(`.${itemClass}:visible`).first();
			}
		}
		if (jqNextActive.length > 0 && jqCurrentActive.get(0) !== jqNextActive.get(0)) {
			jqCurrentActive.removeClass(activeClass);
			jqNextActive.addClass(activeClass).get(0).scrollIntoView({block: 'nearest'});
		}
	},

	onFloatUp(ctl) {
		if (this._open && !zUtl.isAncestor(this, ctl.origin)) {
			this.close();
			this.fire('onBlur');
		}
	},

	onResponse() {
		if (this._redrawItemsFlag) {
			this._redrawItems();
			this._redrawItemsFlag = false;
		}
	},

	onSize() {
		if (this._open) {
			this._repositionPopup();
			this._fixBothScrollbar();
		}
	},

	_repositionPopup() {
		let pp = this.$n('pp');
		if (pp) {
			//check the popup space before position()
			let inp = this.$n(),
				$pp = zk(pp),
				ppHeight = $pp.dimension().height,
				ppWidth = $pp.dimension().width,
				inpDim = zk(inp).dimension(true),
				inpTop = inpDim.top,
				inpLeft = inpDim.left,
				inpHeight = inpDim.height,
				inpWidth = inpDim.width,
				screenX = jq.innerX(),
				screenY = jq.innerY(),
				screenHeight = jq.innerHeight(),
				screenWidth = jq.innerWidth(),
				hPosition = 'start',
				vPosition = 'after',
				opts;

			if (screenX + screenWidth - inpLeft - inpWidth > ppWidth) {
				hPosition = 'start';
			} else if (inpLeft - screenX > ppWidth) {
				hPosition = 'end';
			} else {
				opts = {overflow: true};
			}
			if (screenY + screenHeight - inpTop - inpHeight > ppHeight) {
				vPosition = 'after';
			} else if (inpTop - screenY > ppHeight) {
				vPosition = 'before';
			} else {
				opts = {overflow: true};
			}
			$pp.position(inp, vPosition + '_' + hPosition, opts);
		}
	},
	/**
	 * Drops down or closes the list of items.
	 * @param boolean open
	 */
	setOpen(open: boolean) {
		if (this.isRealVisible()) {
			if (open)
				this.open();
			else
				this.close();
		}
	},
	/**
	 * Returns whether this searchbox is open.
	 *
	 * <p>Default: false.
	 * @return boolean
	 */
	isOpen(): boolean {
		return this._open;
	},

	/**
	 * Drops down the list of items.
	 */
	open() {
		if (this._open) return;

		let pp = this.$n('pp'),
			node = this.$n(),
			zkpp = zk(pp),
			zknode = zk(node);
		jq(node).addClass(this.$s('open'));
		this._open = true;
		this.setFloating_(true, {node: pp});
		this.setFloatZIndex_(pp, this.setTopmost());
		pp.style.minWidth = jq.px0(node.offsetWidth);

		zkpp.makeVParent();
		this._repositionPopup();
		this._fixBothScrollbar();
		let fromBottom = zknode.offsetTopDoubleValue() > zkpp.offsetTopDoubleValue();
		zkpp.slideDown(this, fromBottom ? {anchor: 'b'} : {});
		
		this._focusByWidget = true;
		zk(this.$n('search')).focus();
		this.fire('onOpen', {open: true});
	},
	_fixBothScrollbar() {
		let zknode = zk(this.$n('cave'));
		// In non-WebKit browsers, the vertical scrollbar takes content space (overlay)
		if (zknode.hasHScroll() && zknode.hasVScroll()) {
			let jqpp = jq(this.$n('pp'));
			jqpp.width(jqpp.outerWidth()! + jq.scrollbarWidth());
		}
	},
	/**
	 * Closes the list of items.
	 */
	close(shallFocusWidget?: boolean) {
		if (!this._open) return;
		if (shallFocusWidget)
			this._focusByWidget = true;

		if (this._multiple) {
			if (Object.keys(this._dirtyUuids).length)
				this.fire('onSelect', {selectedUuids: this._dirtyUuids});
		} else {
			let selItem = this._lastSelectedItem;
			if (selItem) {
				let selected = jq(selItem).hasClass(this.$s('selected'));
				this.fire('onSelect', {
					selectedUuids: {[selItem.id]: selected},
					singleSelection: true
				});
			}
		}
		this._dirtyUuids = {};
		this._lastSelectedItem = null;

		jq(this.$n()).removeClass(this.$s('open'));
		this._open = false;
		zk(this.$n('pp')).undoVParent();
		this.setFloating_(false);
		this.fire('onOpen', {open: false});
		if (shallFocusWidget)
			this.$n().focus();
	},

	_redrawpp(out: zk.Buffer) {
		let uuid = this.uuid;
		out.push('<div id="', uuid, '-pp" class="', this.$s('popup'), ' ', this.$s('shadow'), '">');
		out.push('<input id="', uuid, '-search" class="', this.$s('search'),
			'" type="text" autocomplete="off" placeholder="', this._searchMessage, '">');
		out.push('<ul id="', uuid, '-cave" class="', this.$s('cave'), '">');
		if (!this._renderByServer) {
			this._renderItems(out, this._items);
		}
		out.push('</ul></div>');
	},
	_renderItems(out: zk.Buffer, items: Item[]) {
		let selectedUuids = this._selectedUuids,
			selectedClass = this.$s('selected'),
			itemClass = this.$s('item'),
			itemCheckClass = this.$s('item-check'),
			isMultiple = this._multiple;
		items.forEach((it: Item) => {
			let uuid = it.id,
				selected = selectedUuids.indexOf(uuid) != -1;
			out.push('<li id="', uuid, '" class="', itemClass, (selected ? ` ${selectedClass}` : ''), '">');
			if (isMultiple)
				out.push('<i class="', itemCheckClass, '"></i>');
			out.push(it.content, '</li>');
		});
	},
	_redrawItems() {
		let out = new zk.Buffer();
		this._renderItems(out, this._items);
		jq(this.$n('cave')).html(out.join(''));
		if (this._open) {
			this._repositionPopup();
			this._fixBothScrollbar();
		}
	},
	// For server to modify
	_updateItems(content: Item[] | null, partial: boolean, type: UpdateType, start: number, end: number) {
		let cave = this.$n('cave'),
			out = new zk.Buffer(),
			items: Item[] = this._items;

		if (cave) {
			let jqCave = jq(cave),
				jqItems = jqCave.children();
			if (type != UpdateType.INTERVAL_REMOVED)
				this._renderItems(out, content || items);
			if (partial) {
				switch (type) {
					case UpdateType.INTERVAL_ADDED:
						var newContent = out.join('');
						if (start >= jqItems.length)
							jqCave.append(newContent);
						else
							jqItems.eq(start).before(newContent);
						if (content != null)
							items.splice(start, 0, ...content);
						break;
					case UpdateType.INTERVAL_REMOVED:
						for (let i = start; i <= end; i++) {
							cave.removeChild(jqItems.get(i));
						}
						items.splice(start, end - start + 1);
						break;
					case UpdateType.CONTENTS_CHANGED:
						jqItems.eq(start).before(out.join(''));
						for (let i = start; i <= end; i++) {
							cave.removeChild(jqItems.get(i));
						}
						if (content != null)
							items.splice(start, end - start + 1, ...content);
						break;
					default:
				}
			} else {
				cave.innerHTML = out.join('');
			}
		}
	},
	_onSearching(timeout: number) {
		if (this.desktop) {
			let inp = this.$n('search') as HTMLInputElement,
				val = inp.value;
			if (this._lastSearching != val) {
				if (this.isListen('onSearching')) {
					this.fire('onSearching', {
						value: val,
						oldValue: this._lastSearching,
						start: zk(inp).getSelectionRange()[0]
					}, timeout || 5);
				}
				this._lastSearching = val;
				if (!this._renderByServer) {
					this._filterItems(val);
					if (this._open) {
						this._repositionPopup();
						this._fixBothScrollbar();
					}
				}
			}
		}
	},
	_filterItems(term: string) {
		let cave = this.$n('cave'),
			items = jq(cave).children(`.${this.$s('item')}`);
		if (term.length > 0) {
			items.each(function (this: HTMLElement) {
				if (this.innerText.toUpperCase().indexOf(term.toUpperCase()) != -1) {
					this.style.display = '';
				} else {
					this.style.display = 'none';
				}
			});
		} else {
			items.show();
		}
	},
	/**
	 * Clears the selection.
	 * @since 9.5.0
	 */
	clearSelection() {
		if (this._selectedUuids.length > 0) {
			this._selectedUuids = [];
			this._dirtyUuids = {};
			this._lastSelectedItem = null;
			this.fire('onSelect', {selectedUuids: {}, singleSelection: true});
			const selectedClass = this.$s('selected');
			jq(this.$n('pp')).find(`.${selectedClass}`).removeClass(selectedClass);
			this._toggleClearButtonVisible(false);
		}
	},
	_toggleClearButtonVisible(visible: boolean) {
		jq(this.$n('clear')).toggle(visible);
	},
	_focusinContent() {
		jq(this.$n()).addClass(this.$s('focus'));
		if (this._focusByWidget)
			this._focusByWidget = false;
		else
			this.fire('onFocus');
	},
	_focusoutContent() {
		jq(this.$n()).removeClass(this.$s('focus'));
		if (!this._focusByWidget)
			this.fire('onBlur');
	},
	_doSearchFocus() {
		this._skipSearchBlur = false; // reset flag
		if (this._focusByWidget)
			this._focusByWidget = false;
	},
	_doSearchBlur() {
		if (this._skipSearchBlur) {
			this._skipSearchBlur = false;
			return;
		}
		if (!this._focusByWidget)
			this.fire('onBlur');
	},
	_doItemMouseDown() {
		this._skipSearchBlur = true; //to prevent search blur fire event
	}
}, {
	onSearchingDelay: 350,
	onSearchingForced: true,
	_startOnSearching(wgt: zk.Widget) {
		this._stopOnSearching(wgt);
		wgt._tidSearching = setTimeout(wgt.proxy(wgt._onSearching), this.onSearchingDelay);
	},
	_stopOnSearching(wgt: zk.Widget, onBlur?: boolean) {
		if (wgt._tidSearching) {
			clearTimeout(wgt._tidSearching);
			wgt._tidSearching = null;
		}
		if (onBlur) {
			if (this.onSearchingForced && wgt.isListen('onSearching')) {
				wgt._onSearching(-1); //force
			}
		}
	}
});

})();
