/* Linelayout.ts

		Purpose:
		
		Description:
		
		History:
				Mon Oct 14 11:56:04 CST 2019, Created by leon

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/

(() => {

enum UpdateArea {
	FIRST,
	LAST,
}
/**
 * A linelayout component for stylistically displaying chronological information.
 * <p>Only works for browsers supporting CSS flexbox (IE11+, Edge, Chrome, Firefox, Safari).
 * <p>Default {@link #getZclass}: z-linelayout.
 *
 * @since 9.0.0
 */
zkmax.layout.Linelayout = zk.$extends(zul.Widget, {
	_orient: 'vertical',
	_lineStyle: '',
	_firstScale: 1,
	_lastScale: 1,
	
	$define: {
		/**
		 * Returns the orient.
		 * <p>Default: "vertical".
		 * @return String
		 */
		/**
		 * Sets the orient.
		 * @param String orient either horizontal or vertical
		 */
		orient() {
			if (this.desktop) {
				this._updateOrient();
			}
		},
		/**
		 * Returns the CSS style for the line.
		 * @return String
		 */
		/**
		 * Sets the CSS style for the line.
		 * @param String lineStyle the CSS style for the line.
		 */
		lineStyle(v: string) {
			if (this.desktop) {
				this._updateLineStyle();
			}
		},
		/**
		 * Returns the scale of space occupied by the first area.
		 * <p>Default: 1.
		 * @return int
		 */
		/**
		 * Sets the scale of space occupied by the first area.
		 * @param int firstScale the scale of space occupied by the first area.
		 */
		firstScale(v: number) {
			if (this.desktop) {
				this._updateScale(UpdateArea.FIRST);
			}
		},
		/**
		 * Returns the scale of space occupied by the last area.
		 * <p>Default: 1.
		 * @return int
		 */
		/**
		 * Sets the scale of space occupied by the last area.
		 * @param int lastScale the scale of space occupied by the last area.
		 */
		lastScale(v: number) {
			if (this.desktop) {
				this._updateScale(UpdateArea.LAST);
			}
		}
	},
	bind_() {
		this.$supers(zkmax.layout.Linelayout, 'bind_', arguments);
		this._updateLineStyle();
		this._updateScale(true);
		this._updateScale(false);
	},
	_updateOrient() {
		const n = this.$n(),
			isVertical = this._orient == 'vertical';

		if (n) {
			jq(n).toggleClass(this.$s('vertical'), isVertical)
				.toggleClass(this.$s('horizontal'), !isVertical);
			for (var w = this.firstChild; w; w = w.nextSibling) {
				w.updateOrient_();
			}
		}
	},
	_updateLineStyle() {
		const jqLine = jq(this.$n('line'));
		if (jqLine) {
			zk(jqLine).clearStyles();
			jqLine.css(jq.parseStyle(this._lineStyle));
		}
	},
	_updateScale(area: UpdateArea) {
		let first = area == UpdateArea.FIRST;
		jq(this.$n(first ? 'first' : 'last')).css(zk.ie ? '-ms-flex-positive' : 'flex-grow', (first ? this._firstScale : this._lastScale) + '');
		for (var w = this.firstChild; w; w = w.nextSibling) {
			w.updateScale_();
		}
	},
	domClass_() {
		return this.$supers('domClass_', arguments) + ' ' + this.$s(this._orient);
	},
	removeChildHTML_(child) {
		if (this.desktop) {
			jq(child.$n()).remove();
			jq(child.$n('first')).remove();
			jq(child.$n('last')).remove();
		}
		this.$supers('removeChildHTML_', arguments);
	}
});

})();