/*global JQuery*/
/* Anchornav.ts

	Purpose:

	Description:

	History:
		Thu Apr 25 11:19:05 CST 2019, Created by jameschu

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
(() => {
/**
 * An anchornav widget.
 *
 * @author jameschu
 * @since 9.0.0
 */
zkmax.nav.Anchornav = zk.$extends(zul.Widget, {
    _positionFixed: true,
    $define: {
        /**
         * Returns whether it would be fixed position when out of current viewport.
         * @return boolean
         */
        /**
         * Set whether to enable position fix when out of current viewport.
         * @param boolean positionFixed enable or disable
         */
        positionFixed(positionFixed: boolean) {
            if (this.desktop) {
                this._handlePosition();
            }
        }
    },
    bind_() {
        this.$supers(zkmax.nav.Anchornav, 'bind_', arguments);
        zWatch.listen({onScroll: this});
        const anchornavTarget = jq('*[data-anchornav-scroll="' + this._name + '"]');
        if (anchornavTarget.length == 0) {
            jq(window).on('scroll', {anchorwgt: this}, this.onAnchorTargetScroll);
        }
    },
    unbind_() {
        zWatch.unlisten({onScroll: this});
        this.$supers(zkmax.nav.Anchornav, 'unbind_', arguments);
    },

    /**
     * Sets the name.
     * @param String name
     */
    setName(val: string) {
        if (this._name != val) {
            if (this.desktop) {
                let anchornavTarget = jq('*[data-anchornav-scroll="' + this._name + '"]');
                if (anchornavTarget[0])
                    jq(zk(anchornavTarget[0]).$().getCaveNode()).off('scroll', {anchorwgt: this}, this.onAnchorTargetScroll);

                anchornavTarget = jq('*[data-anchornav-scroll="' + val + '"]');
                if (anchornavTarget[0])
                    jq(zk(anchornavTarget[0]).$().getCaveNode()).on('scroll', {anchorwgt: this}, this.onAnchorTargetScroll);
            }
            this._name = val;
        }
    },

    /**
     * Returns the name.
     * @return String
     */
    getName(): string {
        return this._name;
    },

    onScroll() {
        this._handlePosition();
    },

    _handlePosition() {
        const jqn = jq(this.$n()),
            jqCave = jq(this.$n('cave')),
            fixCls = this.$s('fixed'),
            currentOffset = jqCave.offset(),
            currentTop = currentOffset.top,
            currentLeft = currentOffset.left,
            scrollTop = jq(window).scrollTop(),
            isPositionFixedEnabled = this._positionFixed;

        if (jqCave.hasClass(fixCls)) {
            if (!isPositionFixedEnabled || scrollTop < this._oldTop) {
                jqCave.css('left', '');
                jqCave.removeClass(fixCls);
                jqCave.width('');
                this._oldTop = null;
                if (!this.getWidth() && !this.getHflex())
                    jqn.css('width', 'auto');
                if (!this.getHeight() && !this.getVflex())
                    jqn.css('height', 'auto');
            }
        } else if (isPositionFixedEnabled) {
            if ((currentTop - scrollTop) <= 0) {
                this._oldTop = currentTop;
                if (!this.getWidth() && !this.getHflex())
                    jqn.css('width', jqn.width());
                if (!this.getHeight() && !this.getVflex())
                    jqn.css('height', jqn.height());
                jqCave.width(jqn.width());
                jqCave.addClass(fixCls);
                jqCave.css('left', currentLeft);
            }
        }
    },
    onAnchorTargetScroll(event: MouseEvent) {
        const self = event.data.anchorwgt;
        if (!self._activeLinkTarget) {
            const dataAttrStr = 'data-anchornav-target',
                links = jq(self.$n()).find('*[' + dataAttrStr + ']'),
                jqTargets = [],
                scrollTarget = event.target,
                viewTop = (scrollTarget == document) ? jq(scrollTarget).scrollTop() : jq(scrollTarget).offset().top;
            for (let i = 0, l = links.length; i < l; i++) {
                const targetWgt = zk(links.eq(i)).$();
                if (targetWgt.$instanceof(zul.wgt.A) || targetWgt.$instanceof(zul.wgt.Button))
                    jqTargets.push(jq(links.eq(i).attr(dataAttrStr)));
            }
            let activeIndex = -1;
            for (let i = 0, l = jqTargets.length; i < l; i++) {
                const jqTarget = jqTargets[i];
                if (jqTarget[0]) {
                    const targetTop = jqTarget.offset().top;
                    if ((targetTop - viewTop) <= 0)
                        activeIndex = i;
                }
            }
            const activeClz = self.$s('link-active');
            if (self._activeIndex != -1 && activeIndex != self._activeIndex && !links.eq(activeIndex).hasClass(activeClz))
                jq(self.$n()).find('.' + activeClz).removeClass(activeClz);
            self._activeIndex = activeIndex;
            if (activeIndex != -1)
                self.activateLink(links.eq(activeIndex));
        }
    },

    onNavClick(wgt: zk.Widget, targetSelector: string) {
        const self = this,
            targetDom = jq(targetSelector)[0];
        if (!targetDom) {
            zk.error('Anchornav target is not found. (' + targetSelector + ')');
            return;
        }
        self._activeLinkTarget = jq(wgt.$n());
        self.smoothScrollIntoViewTop(targetDom);
        zk.afterAnimate(function () {
            setTimeout(function () {
                self.activateLink(self._activeLinkTarget);
                self._activeLinkTarget = null;
            }, 50); // to prevent async onScroll
        }, -1);
    },

    activateLink(jqLink: JQuery) {
        if (!jqLink) return;
        const activeClz = this.$s('link-active');
        if (!jqLink.hasClass(activeClz))
            jq(this.$n()).find('.' + activeClz).removeClass(activeClz);
        jqLink.addClass(activeClz);
    },

    smoothScrollIntoViewTop(target: Element) {
        if (target) {
            const real = jq('#' + target.id + '-real')[0];
            if (real)
                target = real;
            const targetTop = jq(target).offset().top,
                windowScrollTop = jq(window).scrollTop(),
                totalOffset = targetTop - windowScrollTop;
            if (totalOffset != 0) {
                let doc = window || document.documentElement,
                    offset = totalOffset;
                for (let p = target; (p = p.parentNode) && target != doc; target = p) {
                    offset = this._scrollIntoViewTop(p, target, offset);
                }
            }
        }
    },

    _scrollIntoViewTop(outer, inner: Element, offset: number) {
        if (outer && inner) {
            if (outer == document) return offset;
            let newScrollTop = -1;
            if (outer != document.body) {
                //check if outer has vertical scrollbar
                const borderWidth = Math.round(jq.css(outer, 'borderLeftWidth', true))
                    + Math.round(jq.css(outer, 'borderRightWidth', true)),
                    scrollbarWidth = outer.offsetWidth - borderWidth - outer.clientWidth;
                // Chrome on macOS and some other browser/OS pair could have floating scrollbar whose width will be 0.
                // However, if `outer.scrollHeight > outer.clientHeight`, `outer` can still be scrollable.
                if (scrollbarWidth == 0 && outer.scrollHeight <= outer.clientHeight) return offset;
                const toOuterOffset = jq(inner).offset().top - jq(outer).offset().top;
                if (toOuterOffset == 0) return offset; // in the view top of outer

                newScrollTop = outer.scrollTop + toOuterOffset;
                offset = offset - toOuterOffset;
            } else {
                outer = zk.safari ? 'body' : 'html';
                newScrollTop = jq(outer).scrollTop() + offset;
                const maxScrollTop = jq(document).height() - jq(window).height();
                if (newScrollTop > maxScrollTop) newScrollTop = maxScrollTop;
                offset = 0;
            }
            if (jq(outer)[0].scrollTop != newScrollTop)
                jq(outer).animate({
                    scrollTop: newScrollTop
                }, zk(this).getAnimationSpeed(500));
            return offset;
        }
    }
});
const _xWidget = {};
zk.override(zk.Widget.prototype, _xWidget, {
    bind_(desktop, skipper, after) {
        _xWidget.bind_.apply(this, arguments);
        if (!this.z_virnd) {
            //scroll
            let anchorNavName;
            if (anchorNavName = jq(this).attr('data-anchornav-scroll')) {
                const jqAnchornav = jq('@anchornav');
                for (let i = 0, l = jqAnchornav.length; i < l; i++) {
                    let wgt = zk(jqAnchornav.eq(i)).$();
                    if (wgt.getName() != anchorNavName) continue;
                    jq(this.getCaveNode()).on('scroll', {anchorwgt: wgt}, wgt.onAnchorTargetScroll);
                    jq(window).off('scroll', {anchorwgt: wgt}, wgt.onAnchorTargetScroll);
                }
            }

            //find link (only support <a> and <button>)
            if (jq(this).attr('data-anchornav-target') && (this.$instanceof(zul.wgt.A) || this.$instanceof(zul.wgt.Button))) {
                for (let wgt = this; wgt; wgt = wgt.parent) {
                    if (zkmax.nav.Anchornav.isInstance(wgt))  //check is under anchornav
                        this.domListen_(this.$n(), 'onClick', '_doAnchornavClick', wgt);
                }
            }
        }
    },
    unbind_: function (skipper, after, keepRod) {
        if (!this.z_virnd) {
            //scroll
            let anchorNavName;
            if (anchorNavName = jq(this).attr('data-anchornav-scroll')) {
                const jqAnchornav = jq('@anchornav');
                for (let i = 0, l = jqAnchornav.length; i < l; i++) {
                    let wgt = zk(jqAnchornav.eq(i)).$();
                    if (wgt.getName() != anchorNavName) continue;
                    jq(this.getCaveNode()).off('scroll', {anchorwgt: wgt}, wgt.onAnchorTargetScroll);
                }
            }
            if (jq(this).attr('data-anchornav-target') && (this.$instanceof(zul.wgt.A) || this.$instanceof(zul.wgt.Button))) {
                for (let wgt = this; wgt; wgt = wgt.parent) {
                    if (zkmax.nav.Anchornav.isInstance(wgt)) //check is under anchornav
                        this.domUnlisten_(this.$n(), 'onClick', '_doAnchornavClick', wgt);
                }
            }
        }
        _xWidget.unbind_.apply(this, arguments);
    },

    _doAnchornavClick(event: zk.Event, wgt: zk.Widget) {
        wgt.onNavClick(this, jq(this).attr('data-anchornav-target'));
    }
});
})();

