/* Multislider.ts

	Purpose:

	Description:

	History:
		Thu Sep 12 11:19:05 CST 2019, Created by jameschu

Copyright (C) 2019 Potix Corporation. All Rights Reserved.

This program is distributed under LGPL Version 2.1 in the hope that
it will be useful, but WITHOUT ANY WARRANTY.
*/
(() => {
let _zkf: () => void;
/**
 * A Multislider.
 *
 * @author jameschu
 * @since 9.0.0
 */
zkmax.slider.Multislider = zk.$extends(zkex.slider.Rangeslider, {
    buttonsFireChangeBlocked_: false,
    _btnsRangeValueChangeEvts: {},

    $define: {
        /**
         * Returns the minimum value of the multislider.
         * <p>Default: 0.
         * @return int
         */
        /**
         * Sets the minimum value of the multislider.
         * @param int minimum value
         */
        min: _zkf = function () {
            if (this.desktop)
                this.redrawMarks_();
        },
        /**
         * Returns the maximum value of the multislider.
         * <p>Default: 100.
         * @return int
         */
        /**
         * Sets the maximum value of the multislider.
         * @param int maximum value
         */
        max: _zkf
    },

    bind_() {
        this.$supers(zkmax.slider.Multislider, 'bind_', arguments);
        //redraw marks
        this.redrawMarks_();
    },
    unbind_() {
        this.$supers(zkmax.slider.Multislider, 'unbind_', arguments);
    },

    doMouseDown_(evt: zk.Event) {
        if (this._disabled) return;
        const fc = this.firstChild;
        if (!fc) return;
        let jqTargetDom = jq(evt.domTarget),
            isMarkReached = jqTargetDom.hasClass(this.$s('mark-label'));

        if (isMarkReached || jqTargetDom.hasClass(this.$s('track')) || jqTargetDom.hasClass(fc.$s('area'))) {
            const isVert = this.isVertical(),
                ofDim = isVert ? 'top' : 'left',
                totalDim = this.getSliderSize_();
            let realOf, realOfPx, newVal;
            if (isMarkReached) {
                const jqMark = jqTargetDom.parent(),
                    step = this._step,
                    markVal = jqMark.data('value');
                if ((markVal - this._min) % step == 0) {
                    realOf = parseInt(jqMark[0].style[ofDim]);
                    newVal = markVal;
                } else
                    isMarkReached = false;
            }

            if (!isMarkReached) {
                realOfPx = (isVert ? evt.pageY : evt.pageX) - zk(this.$n('track')).revisedOffset()[isVert | 0];
                realOf = 100 * realOfPx / totalDim;
            }

            // find target button
            let dist, targetWgt, targetBtn, isStartBtn;
            for (let w = fc; w; w = w.nextSibling) {
                let startBtn = w.$n('start-btn'),
                    endBtn = w.$n('end-btn'),
                    distToStartBtn = realOf - parseInt(startBtn.style[ofDim]),
                    distToEndBtn = realOf - parseInt(endBtn.style[ofDim]),
                    absDistToStartBtn = Math.abs(distToStartBtn),
                    absDistToEndBtn = Math.abs(distToEndBtn),
                    isStartCloser = true;
                
                if (distToStartBtn == 0 || distToEndBtn == 0) //if on the clicked pos -> skip
                    return;

                if (distToStartBtn == distToEndBtn) {
                    if (distToStartBtn > 0) // click on the right
                        isStartCloser = false;
                } else {
                    if (distToStartBtn == 0)
                        isStartCloser = false;
                    else if (distToEndBtn != 0)
                        isStartCloser = absDistToStartBtn < absDistToEndBtn;
                }
                const closerBtn = isStartCloser ? startBtn : endBtn,
                    closerDist = isStartCloser ? absDistToStartBtn : absDistToEndBtn;

                let shallMoveFirst = false;
                if (closerDist == dist) { //check inner buttons
                    if (isStartCloser && ((isStartBtn && distToStartBtn > 0) || (!isStartBtn && distToStartBtn < 0)))
                        shallMoveFirst = true;
                    else if (!isStartCloser && ((!isStartBtn && distToEndBtn < 0) || (isStartBtn && distToEndBtn > 0)))
                        shallMoveFirst = true;
                }
                if (!dist || closerDist < dist || shallMoveFirst) {
                    targetWgt = w;
                    targetBtn = closerBtn;
                    dist = closerDist;
                    isStartBtn = (targetBtn == startBtn);
                }
            }
            let oldVals = targetWgt.getValues().slice();
            if (isMarkReached)
                targetWgt._values[isStartBtn ? 0 : 1] = newVal;
            else
                realOf = 100 * targetWgt._realSlidePos(targetBtn, [realOfPx, realOfPx], false)[isVert | 0] / totalDim;
            targetBtn.style[ofDim] = realOf + '%';
            targetWgt.syncArea();
            this.fireValuesChange_(targetWgt, oldVals, targetWgt.getValues());
            // change active target to button
            this.triggerBtnMouseDown_(evt.domEvent, targetWgt, targetBtn, targetBtn == targetWgt.$n('start-btn'));
        }
    },

    redrawMarks_() {
        //clear
        const jqMarks = jq(this.$n('marks'));
        jqMarks.html('');
        const marks = this.getMarks();
        if (!marks) return;
        const marksInfo = [],
            min = this.getMin(),
            max = this.getMax();
        if (!isNaN(parseInt(marks))) { // pure number string
            let markStep = parseInt(marks),
                markVal = min;
            while (markVal <= max) {
                marksInfo.push([markVal, markVal]);
                markVal += markStep;
            }
            if (markVal < (max + markStep))
                marksInfo.push([max, max]);
        } else {
            for (var markVal in marks)
                marksInfo.push([markVal, marks[markVal]]);
        }
        const isVert = this.isVertical(),
            offStr = isVert ? 'top' : 'left',
            labelCls = this.$s('mark-label'),
            markCls = this.$s('mark'),
            sliderValue = max - min,
            l = marksInfo.length;

        for (let i = 0; i < l; i++) {
            if (marksInfo[i][1] - min < 0) continue; //skip invalid
            const jqMark = jq('<div></div>'),
                jqLabel = jq('<div></div>'),
                markValue = parseInt(marksInfo[i][0]);
            jqLabel.addClass(labelCls);
            jqLabel.html(marksInfo[i][1]);
            jqMark.addClass(markCls);
            jqMark.css(offStr, ((markValue - min) / sliderValue * 100) + '%');
            jqMark.data('value', markValue);
            jqMark.append(jqLabel);
            jqMarks.append(jqMark);
        }
        this.updateActiveMarks_();
    },

    fixOtherChildrenValues_(wgt: zk.Widget) {
        let values = wgt.getValues(),
            btnDimStr = this.isVertical() ? 'top' : 'left',
            beforeBtns = true,
            btnsEvts = this._btnsRangeValueChangeEvts,
            firstBtnOff = wgt.$n('start-btn').style[btnDimStr],
            endBtnOff = wgt.$n('end-btn').style[btnDimStr];

        for (let w = this.firstChild; w; w = w.nextSibling) {
            let vals = w.getValues(),
                startBtn = w.$n('start-btn'),
                endBtn = w.$n('end-btn'),
                valuesChanged = false,
                shallFireChange = true;

            if (wgt == w) {
                beforeBtns = false;
                continue;
            } else {
                if (beforeBtns && vals[0] > values[0] || !beforeBtns && vals[0] < values[0]) {
                    w._values[0] = vals[0] = values[0];
                    startBtn.style[btnDimStr] = firstBtnOff;
                    valuesChanged = true;
                    if (vals[0] > vals[1]) { //when dragging
                        w._values[1] = vals[1] = values[0];
                        endBtn.style[btnDimStr] = firstBtnOff;
                    }
                }
                if (beforeBtns && vals[1] < values[1] || !beforeBtns && vals[1] > values[1]) {
                    w._values[1] = vals[1] = values[1];
                    endBtn.style[btnDimStr] = endBtnOff;
                    valuesChanged = true;
                    if (vals[0] > vals[1]) { //when dragging
                        w._values[0] = vals[0] = values[1];
                        startBtn.style[btnDimStr] = endBtnOff;
                    }
                }
            }
            if (valuesChanged) {
                w.syncArea();
                if (shallFireChange)
                    btnsEvts[w.uuid] = vals;
            }
        }
    },

    updateActiveMarks_() {
        //do nothing
    },

    afterBtnsValuesChange_() {
        // update other buttons value
        const btnsEvts = this._btnsRangeValueChangeEvts;
        for (let uuid in btnsEvts)
            zk.$(uuid).fire('onRangeValueChange', {values: btnsEvts[uuid]});
        this._btnsRangeValueChangeEvts = {}; //reset
    }
});
})();