/* Step.js

		Purpose:
                
		Description:
                
		History:
				Thu Sep 12 11:35:49 CST 2019, Created by jameschu

Copyright (C) 2019 Potix Corporation. All Rights Reserved.

*/
(() => {
/**
* A step.
* <p>Only support browsers that support Flex. (IE10+, Edge, Chrome, Firefox, Safari)
* <p>Default {@link #getZclass}: z-step.
*
* <p>Available in ZK EE.
* @author jameschu
* @since 9.0.0
*/
zkmax.wgt.Step = zk.$extends(zul.Widget, {
	_complete: false,
	_error: false,
	$define: {
		/**
		 * Returns the title.
		 * <p>Default: empty.
		 * @return String
		 */
		/**
		 * Sets the title.
		 * @param String title
		 */
		title(v: string) {
			if (this.desktop)
				jq(this.$n('title')).text(v);
		},
		/**
		 * Returns whether this step is complete.
		 * <p>Default: false.
		 * @return boolean whether this step is complete
		 */
		/**
		 * Sets whether this step is complete.
		 * @param boolean complete whether this step is complete
		 */
		complete(v: boolean) {
			if (this.desktop) {
				this._setCompleteDirectly(v);
			}
		},
		/**
		 * Returns whether this step is in error.
		 * <p>Default: false.
		 * @return boolean whether this step is in error
		 */
		/**
		 * Sets whether this step is in error.
		 * @param boolean error whether this step is in error.
		 */
		error(v: boolean) {
			if (this.desktop) {
				jq(this.$n()).toggleClass(this.$s('error'), v);
				this._adjustIconContent();
			}
		},
		/**
		 * Sets the icon font
		 * @param String iconSclass a CSS class name for the icon font
		 */
		/**
		 * Returns the icon font
		 * @return String iconSclass a CSS class name for the icon font
		 */
		iconSclass(v: string) {
			if (this.desktop)
				this._adjustIconContent();
		}
	},
	bind_() {
		this.$supers(zkmax.wgt.Step, 'bind_', arguments);
		this.domListen_(this.$n(), 'onClick', '_activate');
		this._adjustIconContent();
	},
	unbind_() {
		this.$supers(zkmax.wgt.Step, 'unbind_', arguments);
		this.domUnlisten_(this.$n(), 'onClick', '_activate');
	},
	_activate() {
		const stepbar = this.getStepbar();
		if (!stepbar.isLinear())
			stepbar.setActiveIndex(this.getChildIndex());
	},
	_toggleActive(active: boolean) {
		const jqn = jq(this.$n()),
			activeCls = this.$s('active');
		jqn.toggleClass(activeCls, active);
		this._adjustIconContent();
	},
	_setCompleteDirectly(complete: boolean) {
		const jqn = jq(this.$n()),
			completeCls = this.$s('complete');
		jqn.toggleClass(completeCls, complete);
		this._adjustIconContent();
	},
	_adjustIconContent() {
		let nIcon = this.$n('icon'),
			iconSclass = this.getIconSclass(),
			className = this.$s('icon-empty');
		nIcon.className = this.$s('icon'); //reset
		if (iconSclass) {
			className = iconSclass;
		} else if (this.isError()) {
			className = 'z-icon-exclamation';
		} else if (this.isComplete()) {
			className = 'z-icon-check';
		}
		jq(nIcon).addClass(className);
	},
	_isActiveStep() {
		return this.getStepbar()._getActiveStep() == this;
	},
	getStepbar() {
		return this.parent;
	},
	domClass_() {
		let cls = this.$supers('domClass_', arguments);
		if (this.isComplete())
			cls += ' ' + this.$s('complete');
		if (this.isError())
			cls += ' ' + this.$s('error');
		if (this._isActiveStep())
			cls += ' ' + this.$s('active');
		return cls;
	}
});
})();