/* James Chu, Potix, 20210604, this file will generate zkFragmentContentParser.js, please assign 'zkFragmentContentParser' into window (for source map)*/
%lex
%options case-insensitive
%x TAG ANNO IFATTR IFATTRVALUE FOREACHATTR EL TAGEND CMDANNO CMDANNOARGS CMDANNOARGSVAL ENCLOSETAG
%%

\s+																	return 'BLANK';
"="																	return '=';
[\"\']																return 'QUOTE';
'<!--'((?!\-\-\>)[\S\s])*'-->'										return 'BLANK';
"<"																	{
																		this.begin('TAG');
																		return '<';
																	}
">"																	{
																		this.begin('TAGEND');
																		return '>';
																	}
"/"																	return '/';
"@"																	{
																		yy.currentState = 'INITIAL';
																		this.begin('ANNO');
																		return '@';
																	}
"("																	return '(';
")"																	return ')';
"on"[a-zA-Z]+														return 'EVENT';
"textContent"														return 'TEXTCONTENT';
[^="'<]+															return 'STRING';
<<EOF>>																return 'EOF';

<ANNO>\s+															return 'BLANK';
<ANNO>("command"|"global-command")									{
																		this.begin('CMDANNO');
																		return 'COMMAND';
																	}
<ANNO>"load"														return 'LOAD';
<ANNO>"save"														return 'SAVE';
<ANNO>"bind"														return 'BIND';
<ANNO>"jsvalidator"													return 'JSVALIDATOR';
<ANNO>"("															{
																		this.begin('EL');
																		return '(';
																	}
<ANNO>")"															{
																		this.begin(yy.currentState);
																		delete yy.currentState;
																		return ')';
																	}

<CMDANNO>\s+														return 'BLANK';
<CMDANNO>"("														return '(';
<CMDANNO>")"														{
																		this.begin('INITIAL');
																		return ')';
																	}
<CMDANNO>[\'\"]														return 'QUOTE';
<CMDANNO>","														{
																		this.begin('CMDANNOARGS');
																		return 'FIRSTCOMMA';
																	}
<CMDANNO>"="														return '=';
<CMDANNO>[a-zA-Z0-9]+												return 'CMDMETHOD';

<CMDANNOARGS>\s+													return 'BLANK';
<CMDANNOARGS>[\'\"]												    return 'QUOTE';
<CMDANNOARGS>"="													{
																		this.begin('CMDANNOARGSVAL');
																		return '=';
																	}
<CMDANNOARGS>","													return ',';
<CMDANNOARGS>")"													{
																		this.begin('INITIAL');
																		return ')';
																	}
<CMDANNOARGS>[a-zA-Z0-9]+											return 'ARGKEY';

<CMDANNOARGSVAL>\s+													return 'BLANK';
<CMDANNOARGSVAL>[\'\"]												return 'QUOTE';
<CMDANNOARGSVAL>'event'(\.[^,)]+)?									{
																		this.begin('CMDANNOARGS');
																		return 'event';
																	}
<CMDANNOARGSVAL>[^,)]+  							        		{
																		this.begin('CMDANNOARGS');
																		return 'ARGVAL';
																	}

<TAG>\s+															return 'BLANK';
<TAG>"/"															{
																		this.begin('ENCLOSETAG');
																		return '/';
																	}
<TAG>"if"															{
																		yy.currentTag = yytext;
																		this.begin('IFATTR');
																		return 'IF';
																	}
<TAG>"forEach"														{
																		yy.currentTag = yytext;
																		if (yy.foreachLayer == undefined)
																			yy.foreachLayer = 0;
																		yy.foreachLayer++;
																		if (yy.foreach == undefined)
																			yy.foreach = [];
																		yy.foreach[yy.foreachLayer] = {};
																		if (yy.foreachEach == undefined)
																			yy.foreachEach = [];
																		this.begin('FOREACHATTR');
																		return 'FOREACH';
																	}
<TAG>[a-zA-Z0-9\-]+													{
																		yy.currentTag = yytext;
																		this.begin('INITIAL');
																		return 'TAGNAME';
																	}
<TAG>"<"															return '<';

<ENCLOSETAG>\s+														return 'BLANK';
<ENCLOSETAG>"if"													{
																		yy.currentTag = yytext;
																		return 'IF';
																	}
<ENCLOSETAG>"forEach"												{
																		yy.currentTag = yytext;
																		return 'FOREACH';
																	}
<ENCLOSETAG>[a-zA-Z0-9\-]+											{
																		yy.currentTag = yytext;
																		return 'TAGNAME';
																	}
<ENCLOSETAG>">"														{
																		this.begin('TAGEND');
																		return '>';
																	}

<IFATTR>\s+															return 'BLANK';
<IFATTR>"test"														return 'TEST';
<IFATTR>[a-zA-Z0-9\-]+												return 'INVALIDATTR';
<IFATTR>"="															{
																		this.begin('IFATTRVALUE');
																		return '=';
																	}
<IFATTR>">"															{
																		this.begin('TAGEND');
																		return '>';
																	}

<IFATTRVALUE>\s+													return 'BLANK';
<IFATTRVALUE>"@load("												return 'LOAD';
<IFATTRVALUE>")"													{
																		this.begin('INITIAL');
																		return ')';
																	}
<IFATTRVALUE>[\"\']													return 'QUOTE';
<IFATTRVALUE>[a-zA-Z0-9+\-*\/!=><().\s]+(?=\)\")					return 'CONDITION';

<FOREACHATTR>\s+													return 'BLANK';
<FOREACHATTR>"items"												{
																		yy.foreachattr = 'items';
																		return 'ITEMS';
																	}
<FOREACHATTR>"var"													{
																		yy.foreachattr = 'variable';
																		return 'VAR';
																	}
<FOREACHATTR>"begin"												{
																		yy.foreachattr = 'begin';
																		return 'BEGIN';
																	}
<FOREACHATTR>"end"													{
																		yy.foreachattr = 'end';
																		return 'END';
																	}
<FOREACHATTR>"step"													{
																		yy.foreachattr = 'step';
																		return 'STEP';
																	}
<FOREACHATTR>"varStatus"											{
																		yy.foreachattr = 'varStatus';
																		return 'VARSTATUS';
																	}
<FOREACHATTR>"="													return '=';
<FOREACHATTR>[\"\']													return 'QUOTE';
<FOREACHATTR>"@"													{
																		yy.currentState = 'FOREACHATTR';
																		this.begin('ANNO');
																		return '@';
																	}
<FOREACHATTR>">"													{
																		delete yy.foreachattr;
																		this.begin('TAGEND');
																		return '>';
																	}
<FOREACHATTR>[a-zA-Z0-9-\s:#;]+										return 'STRING';

<EL>[\w.'\[\]]+(?=\))												{
																		this.begin('ANNO');
																		return 'EL';
																	}

<TAGEND>'<!--'((?!\-\-\>)[\S\s])*'-->'								return 'TEXTNODE';
<TAGEND>'<'															{
																		this.begin('TAG');
																		return '<';
																	}
<TAGEND>[^><]+														return 'TEXTNODE';
<TAGEND><<EOF>>														return 'EOF';

/lex

%{
	function log(text){
		//console.log(text);
	}

	function enclosing(yy, arg){
		var r;
		if (yy.currentTag == 'forEach') {
			r = arg + '{{forEach\$\$Status(\'\', '
			+ yy.foreachLayer
			+ ', \'\')}}';
			delete yy.foreachEach[yy.foreachLayer];
			yy.foreachLayer--;
		} else {
			r = arg;
		}
		return r;
	}
	function foreachending(yy, arg) {
		var foreach = yy.foreach[yy.foreachLayer],
			r = ' v-for="('
			+ (foreach.variable || 'each') + ', index) in '
			+ 'zk\$\$forEach(' + foreach.items
			+ ', '
			+ yy.foreachLayer
			+ ', '
			+ (foreach.begin || '\'\'')
			+ ', '
			+ (foreach.end || '\'\'')
			+ ', '
			+ (foreach.step || '\'\'')
			+ ')"'
			+ arg
			+ '{{forEach\$\$Status('
			+ (foreach.variable || 'each')
			+ ', '
			+ yy.foreachLayer
			+ ', index)}}';
		var each = yy.foreachEach[yy.foreachLayer] = {};
		each[foreach.variable || 'each'] = foreach.items;
		return r;
	}
%}

%left '=' '<' '>' '/' 'QUOTE' '(' 'BLANK' '@' 'ITEMS' 'VAR' 'BEGIN' 'END' 'STEP' 'VARSTATUS' 'CMDMETHOD' 'ARGKEY' 'ARGVAL' ','
%right ')'

%start expressions

%%

expressions
	: e EOF
		{ log($1); return $1;}
	;

e
	: BLANK e
		{ $$ = $1 + $2; }
	| '<' tag
		{ $$ = $1 + $2; }
	| '/' '>' e
		{ $$ = $1 + $2 + $3; }
	| '>' e
		{ $$ = $1 + $2; }
	| TEXTNODE e
		{ $$ = $1 + $2; }
	| STRING e
		{ $$ = $1 + $2; }
	|
		{ $$ = ''; }
	;

tag
	: BLANK tag
		{ $$ = $1 + $2; }
	| IF if
		{ $$ = 'template' + $2; }
	| FOREACH foreach
		{ $$ = 'template' + $2; }
	| INPUT inputattrs
		{ $$ = $1 + $2; }
	| TAGNAME attrs
		{ $$ = $1 + $2; }
	| enclose
		{ $$ = $1; }
	;

enclose
	: '/' encloseing '<' tag
		{ $$ = $1 + $2 + $3 + $4; }
	| '/' encloseing
		{ $$ = $1 + $2; }
	;

encloseing
	: BLANK encloseing
		{ $$ = $1 + $2; }
	| IF encloseing
		{ $$ = 'template' + $2; }
	| FOREACH encloseing
		{ $$ = 'template' + $2; }
	| INPUT encloseing
		{ $$ = $1 + $2; }
	| TAGNAME encloseing
		{ $$ = $1 + $2; }
	| '>' anything
		{
			log('yoyoyoyoyoy');
			$$ = enclosing(yy, $1) + $2;
		}
	| '>'
		{
			log('yayay');
			$$ = enclosing(yy, $1);
		}
	;

anything
	: TEXTNODE
		{ $$ = $1; }
	| TEXTNODE anything
		{ $$ = $1 + $2; }
	;

if
	: BLANK if
		{ $$ = $1 + $2; }
	| TEST '=' iftestValue '>' e
		{ $$ = 'v-if' + $2 + $3 + $4 + $5; }
	| INVALIDATTR '=' iftestValue '>' e
		{ $$ = $1 + ' attribute is not suppoted for if element'; }
	;

iftestValue
	: BLANK iftestValue
		{ $$ = $1 + $2; }
	| QUOTE ifcondition QUOTE
		{ $$ = $1 + $2 + $3; }
	;

ifcondition
	: BLANK ifcondition
		{ $$ = ($1 + $2).trim(); }
	| LOAD ifcondition
		{ $$ = $2; }
	| CONDITION ')'
		{
			var one = $1;
			if (yy.foreachLayer) {
				var regex = new RegExp('(' + yy.foreach[yy.foreachLayer].varStatus + '\.)(?:first|last|begin|end|step|each|current|index|count)', 'g');
				var match;
				while (match = regex.exec(one)) {
					one = one.replace(match[1], 'zk\$\$\$\$forEachStatus[' + (yy.foreachLayer - 1) + '].');
				}
			}
			$$ = one;
		}
	;

foreach
	: BLANK foreach
		{ $$ = $2; }
	| '=' foreach
		{ $$ = $2; }
	| QUOTE foreachvalue QUOTE foreach
		{ $$ = $4; }
	| ITEMS foreach
		{ $$ = $2; }
	| VAR foreach
		{ $$ = $2; }
	| BEGIN foreach
		{ $$ = $2; }
	| END foreach
		{ $$ = $2; }
	| STEP foreach
		{ $$ = $2; }
	| VARSTATUS foreach
		{ $$ = $2; }
	| foreachend
		{ $$ = $1; }
	;

foreachend
	: foreachending '<' tag
		{ $$ = $1 + $2 + $3; }
	;

foreachending
	: '>'
		{
			log('foreachending1');
			$$ = foreachending(yy, $1);
		}
	| '>' anything
		{
			log('foreachending2');
			$$ = foreachending(yy, $1) + $2;
		}
	| foreachending BLANK
		{ $$ = $1 + $2; }
	;

foreachvalue
	: BLANK foreachvalue
		{ $$ = $2; }
	| '@' foreachvalue
		{ $$ = $2; }
	| LOAD foreachvalue
		{ $$ = $2; }
	| BIND foreachvalue
		{ $$ = $2; }
	| '(' foreachvalue
		{ $$ = $2; }
	| EL foreachvalue
		{ yy.foreach[yy.foreachLayer][yy.foreachattr] = $1; }
	| ')'
	| STRING
		{ yy.foreach[yy.foreachLayer][yy.foreachattr] = $1; }
	|
	;

inputattrs
	: BLANK inputattrs
		{ $$ = $1 + $2; }
	| '=' inputattrs
		{
			if (yy.skip) {
				yy.skip = false;
				$$ = $2;
			} else {
				$$ = $1 + $2;
			}
		}
	| QUOTE inputvalue QUOTE inputattrs
		{
			if ($4.includes('@change') && $2.includes('command')) {
				$$ = $4;
				yy.skip = true;
			} else {
				$$ = $1 + $2 + $3 + $4;
			}
		}
	| VALUE inputattrs
		{ $$ = 'v-model.lazy' + $2; }
	| ONCHANGE inputattrs
		{ $$ = $2; }
	| EVENT inputattrs
		{ $$ = '@' + $1.substring(2).toLowerCase() + $2; }
	| STRING inputattrs
		{ $$ = $1 + $2; }
	;

inputvalue
	: BLANK inputvalue
		{ $$ = $1 + $2; }
	| '@' inputvalue
		{ $$ = $2; }
	| LOAD inputvalue
		{ $$ = $2; }
	| BIND inputvalue
		{
			var inputValue = yy.inputValue,
				foreachLayer = yy.foreachLayer;
			if (inputValue) {
				var bindProp = $2.split('.')[0];
				if (foreachLayer) {
					for (foreachLayer; foreachLayer > 0; foreachLayer--) {
						var map = yy.foreachEach[foreachLayer];
						if (map[bindProp]) {
							bindProp = map[bindProp];
						}
					}
				}
				yy.inputBindingData = bindProp;
			}
			$$ = $2;
		}
	| SAVE inputvalue
		{ $$ = $2; }
	| COMMAND inputvalue
	| '(' inputvalue
		{ $$ = $2; }
	| EL inputvalue
		{ $$ = $1 + $2; }
	| QUOTE inputvalue
		{ $$ = $1 + $2; }
	| FIRSTCOMMA inputvalue
		{
			if (yy.inputOnChange) {
				yy.inputChangeEventBindingArgs = $2;
			}
			$$ = $1 + $2;
		}
	| ',' inputvalue
		{
			if (yy.inputOnChange) {
				yy.inputChangeEventBindingArgs = $2;
			}
			$$ = $1 + $2;
		}
	| ')'
		{ $$ = ''; }
	| CMDMETHOD inputvalue
		{
			if (yy.inputOnChange) {
				yy.inputChangeEventBinding = $1;
			}
		}
	| ARGKEY inputvalue
		{ $$ = $1 + $2; }
	| '=' inputvalue
		{ $$ = $1 + $2; }
	| ARGVAL inputvalue
		{ $$ = $1 + $2; }
	| STRING
		{
			var inputOnChange = yy.inputOnChange;
			if (inputOnChange) {
				inputOnChange = false;
			}
			$$ = $1;
		}
	;

attrs
	: BLANK attrs
		{ $$ = $1 + $2; }
	| STRING '=' QUOTE loadel QUOTE attrs
		{ $$ = ':' + $1 + $2 + $3 + $4 + $5 + $6; }	
	| STRING '=' QUOTE saveel QUOTE attrs
		{ $$ = '@change="sendEvent($event, \'' + $4 + '\', \'\')"' + $6; }
	| STRING '=' QUOTE saveel BLANK jsvalidatorel QUOTE attrs
		{ $$ = '@change="sendEvent($event, \'' + $4 + '\', \'\')"' + $5 + ':data-validator="' + $6 + '"' + $8; }
	| STRING '=' QUOTE bindel QUOTE attrs
		{ $$ = 'v-model.lazy="zkHolder.' + $4 + '" @change="sendEvent($event, \'' + $4 + '\', \'\')"' + $6; }
	| STRING '=' QUOTE bindel BLANK jsvalidatorel QUOTE attrs
		{ $$ = 'v-model.lazy="zkHolder.' + $4 + '" @change="sendEvent($event, \'' + $4 + '\', \'\')"' + $5 + ':data-validator="' + $6 + '"' + $8; }  
	| STRING '=' QUOTE value QUOTE attrs
		{ $$ = $1 + $2 + $3 + $4 + $5 + $6; }	
	| EVENT '=' QUOTE value QUOTE attrs
		{ $$ = '@' + $1.substring(2).toLowerCase() + $2 + $3 + $4 + $5 + $6; }
	| TEXTCONTENT '=' QUOTE loadel QUOTE attrs
		{ $$ = 'v-text' + $2 + $3 + $4 + $5 + $6; }
	| TEXTCONTENT '=' QUOTE bindel QUOTE attrs
		{ $$ = 'v-text' + $2 + $3 + $4 + $5 + $6; }
	| end
		{ $$ = $1; }
	;

loadel
	: '@' LOAD '(' EL ')'
		{ $$ = $4; }
	;

saveel
	: '@' SAVE '(' EL ')'
		{ $$ = $4; }
	;

bindel
	: '@' BIND '(' EL ')'
		{ $$ = $4; }
	;

jsvalidatorel
	: '@' JSVALIDATOR '(' EL ')'
		{ $$ = $4; }
	;

end
	: ending '<' tag
		{ $$ = $1 + $2 + $3; }
	| ending
		{ $$ = $1; }
	;

ending
	: '>'
		{ $$ = $1; }
	| '>' anything
		{ $$ = $1 + $2; }
	| '/' ending
		{ $$ = $1 + $2; }
	| ending BLANK
		{ $$ = $1 + $2; }
	;

value
	: BLANK value
		{ $$ = $1 + $2; }
	| '@' value
		{ $$ = $2; }
	| COMMAND value
		{ $$ = 'sendEvent($event, \'\', ' + $2 + ')'; }
	| '(' value
		{ $$ = $2; }
	| EL value
		{ $$ = $1; }
	| QUOTE value
		{ $$ = $1 + $2; }
	| FIRSTCOMMA value
		{ $$ = $1 + ' {' + $2.trim() + '}'; }
	| ',' value
		{ $$ = $1 + $2; }
	| ')'
		{ $$ = ''; }
	| CMDMETHOD value
		{ $$ = $1 + $2; }
	| ARGKEY value
		{ $$ = $1 + $2; }
	| '=' value
		{ $$ = ':' + $2; }
	| 'event' value
		{ $$ = '$' + $1 + $2; }
	| ARGVAL value
		{ $$ = $1 + $2; }
	| STRING
		{ $$ = $1; }
	;
